<?php
/**
 * @file
 * Functions to display promo block.
 */

/**
 * Get teaser records for taxonomy and type.
 *
 * @param int $tid
 *   The taxonomy id associated to teaser records to display.
 * @param string $type
 *   The block type to display - primary / secondary.
 *
 * @return array
 *   Teaser objects.
 */
function node_promo_picker_load_teaser_records($tid, $type) {
  $teasers = array();
  // Set type fields.
  $field = ($type == 'primary') ? 'primary_flag' : 'secondary_flag';
  $delta_field = ($type == 'primary') ? 'primary_delta' : 'secondary_delta';
  $limit = ($type == 'primary') ? (int) variable_get('promo_picker_max_primary_teasers', 3) : (int) variable_get('promo_picker_max_secondary_teasers', 3);
  // Create sql.
  $sql = 'SELECT ppt.*, pptt.' . check_plain($delta_field) . ' AS delta FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE ppt.' . check_plain($field) . ' = 1 AND pptt.tid = :tid ORDER BY ' . check_plain($delta_field) . ' ASC LIMIT 0, ' . $limit;
  $result = db_query($sql, array(':tid' => $tid));
  foreach ($result as $teaser) {
    // Get related node.
    $teaser->node = node_load($teaser->related_nid);
    // Only include published nodes.
    if ($teaser->node->status == 1) {
      $teaser->images = array();
      // Get teaser images.
      $images = db_query('SELECT * FROM {promo_picker_teasers_images} WHERE teaser_id = :teaser_id ORDER BY delta ASC', array(':teaser_id' => $teaser->teaser_id));
      foreach ($images as $row) {
        $img = node_load($row->image_nid);
        $teaser->images[] = $img;
      }
      // Add teaser to array.
      $teasers[] = $teaser;
    }
  }
  return $teasers;
}
