<?php
/**
 * @file
 * Administration pages for managing promo block content.
 */

/**
 * Form definition for List admin page.
 */
function node_promo_picker_blocks_admin_list_form($state) {
  $form = array();
  // Get defined taxonomies.
  $vids = variable_get('promo_picker_taxonomy_settings', array());
  if (empty($vids)) {
    drupal_set_message(t('No taxonomies defined'), 'warning');
  }
  // Build table content.
  $header = array(
    'Taxonomy',
    array(
      'data' => 'Primary block',
      'colspan' => 2,
    ),
    array(
      'data' => 'Secondary block',
      'colspan' => 2,
    ),
  );
  foreach ($vids as $vid) {
    $vocab = db_query('Select name FROM {taxonomy_vocabulary} WHERE vid = :vid', array(':vid' => $vid))->fetchField();
    $form['vocabulary_' . $vid] = array(
      '#type' => 'fieldset',
      '#title' => check_plain('Vocabulary: ' . $vocab),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $rows = array();
    $allowed_elements = array(
      'table',
      'tr',
      'td',
      'th',
      'thead',
      'tbody',
      'a',
      'p',
      'div',
      'strong',
    );
    $terms = taxonomy_get_tree($vid);
    foreach ($terms as $t) {
      $rows[] = array(
        $t->name,
        l(t('edit'), 'admin/structure/promo_picker/blocks/edit/' . $t->tid . '/primary'),
        l(t('reset'), 'admin/structure/promo_picker/blocks/reset/' . $t->tid . '/primary'),
        l(t('edit'), 'admin/structure/promo_picker/blocks/edit/' . $t->tid . '/secondary'),
        l(t('reset'), 'admin/structure/promo_picker/blocks/reset/' . $t->tid . '/secondary'),
      );
    }
    $table = theme('table', array('header' => $header, 'rows' => $rows));
    $form['vocabulary_' . $vid]['table'] = array(
      '#markup' => filter_xss($table, $allowed_elements),
    );
  }
  return $form;
}

/**
 * Reset promo block teasers.
 */
function node_promo_picker_blocks_admin_reset() {
  $tid = arg(5);
  $type = arg(6);
  // Set field to update.
  $field = ($type == 'primary') ? 'primary_flag' : 'secondary_flag';
  $alt_field = ($type == 'primary') ? 'secondary_flag' : 'primary_flag';
  // Reset teasers.
  $sql = 'UPDATE {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id SET ppt.' . db_escape_table($field) . ' = 0 WHERE pptt.tid = :tid';
  db_query($sql, array(':tid' => $tid));
  $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE pptt.tid = :tid AND ppt.' . db_escape_table($alt_field) . ' = 0';
  db_query($sql, array(':tid' => $tid));
  drupal_set_message(t('Promo block reset.'));
  // Set page to return to.
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/structure/promo_picker/blocks';
  drupal_goto($goto);
}

/**
 * Form definition for editing promo block.
 *
 * @see node_promo_picker_blocks_admin_edit_form_validate()
 * @see node_promo_picker_blocks_admin_edit_form_submit()
 */
function node_promo_picker_blocks_admin_edit_form($state) {
  $form = array();
  // Get parameters.
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  $order_field = ($block == 'primary') ? 'primary_delta' : 'secondary_delta';
  $term = taxonomy_term_load($tid);
  // Taxonomy block details.
  $form['block_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Promo block details',
  );
  $form['block_panel']['tid'] = array(
    '#type' => 'item',
    '#title' => 'Taxonomy ID',
    '#markup' => $term->tid,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['block_panel']['taxonomy'] = array(
    '#type' => 'item',
    '#title' => 'Taxonomy',
    '#markup' => $term->name,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['block_panel']['placement'] = array(
    '#type' => 'item',
    '#title' => 'Type',
    '#markup' => ucwords($block),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $themes = array('' => 'Default (' . ucwords($block) . ' block)');
  $additional_themes = variable_get('promo_picker_additional_themes', array());
  foreach ($additional_themes as $f => $at) {
    $themes[$f] = $at['title'];
  }
  $theme = variable_get('promo_picker_theme_' . $tid . '_' . $block, '');
  $theme = (array_key_exists($theme, $themes)) ? $theme : '';
  $form['block_panel']['theme'] = array(
    '#type' => 'select',
    '#title' => 'Theme',
    '#options' => $themes,
    '#default_value' => $theme,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  // Add fields to add node to block.
  $form['add_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Add node teaser',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['add_panel']['node'] = array(
    '#type' => 'textfield',
    '#title' => 'Search nodes',
    '#description' => 'Search for node to add teaser for.',
    '#autocomplete_path' => 'admin/structure/promo_picker/node/autocomplete',
  );
  $form['add_panel']['add_teasers'] = array(
    '#type' => 'submit',
    '#value' => 'Add node',
    '#validate' => array('node_promo_picker_add_node_to_block_validate'),
    '#submit' => array('node_promo_picker_add_node_to_block_submit'),
  );
  // Get all nodes displayed on the taxonomy.
  $form['teasers'] = array(
    '#tree' => TRUE,
    '#theme' => 'promo_block_teaser_list',
  );
  $sql = 'SELECT n.title AS node_title, n.nid, ppt.*, pptt.' . check_plain($order_field) . ' AS weight FROM {node} AS n JOIN {promo_picker_teasers} AS ppt ON n.nid = ppt.related_nid JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = :tid AND ' . check_plain($field) . ' = 1 ORDER BY pptt.' . check_plain($order_field) . ' ASC';
  $result = db_query($sql, array(':tid' => $tid));
  $max_teaser = variable_get('promo_picker_max_' . $block . '_teasers', 3);
  $has_teasers = FALSE;
  foreach ($result as $n) {
    // Get all teaser records for current node.
    $other = db_query('SELECT title, teaser_id FROM {promo_picker_teasers} WHERE related_nid = :related_nid', array(':related_nid' => $n->related_nid));
    $options = array();
    foreach ($other as $t) {
      $options[$t->teaser_id] = $t->title;
    }
    $query = array(
      'return_to' => 'admin/structure/promo_picker/blocks/edit/' . $tid . '/' . $block,
      'referer' => ((!empty($_GET['return_to'])) ? $_GET['return_to'] : ''),
    );
    $form['teasers'][$n->teaser_id]['#teaser'] = (array) $n;
    $form['teasers'][$n->teaser_id]['node'] = array(
      '#markup' => $n->node_title . ' (' . $n->nid . ')',
    );
    $form['teasers'][$n->teaser_id]['teaser_select'] = array(
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $n->teaser_id,
    );
    $form['teasers'][$n->teaser_id]['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $n->weight,
      '#delta' => $max_teaser,
    );
    $form['teasers'][$n->teaser_id]['edit'] = array(
      '#markup' => l(t('Edit'),
        'admin/structure/promo_picker/edit-teaser/' . $n->teaser_id,
        array(
          'query' => $query,
          'attributes' => array(
            'title' => 'Edit teaser',
          ),
        )
      ),
    );
    $form['teasers'][$n->teaser_id]['new'] = array(
      '#markup' => l(t('New'),
        'admin/structure/promo_picker/create-teaser/' . $n->related_nid,
        array(
          'query' => $query,
          'attributes' => array(
            'title' => 'Create new teaser for node',
          ),
        )
      ),
    );
    $form['teasers'][$n->teaser_id]['remove'] = array(
      '#markup' => l(t('Remove'),
        'admin/structure/promo_picker/blocks/remove-teaser/' . $n->teaser_id . '/' . $tid . '/' . $block,
        array(
          'query' => $query,
          'attributes' => array(
            'title' => 'Remove teaser from promo block',
          ),
        )
      ),
    );
    $has_teasers = TRUE;
  }
  if ($has_teasers) {
    $form['save'] = array(
      '#type' => 'submit',
      '#value' => t('Save changes'),
    );
  }
  return $form;
}

/**
 * Validate add node form submission.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_add_node_to_block_validate($form, &$state) {
  // Get parameters.
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  // Validate node field.
  $nid = 0;
  if (preg_match('/\([0-9]+\)$/', $state['values']['node'], $match)) {
    $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
    $exists = db_query('SELECT nid FROM {node} WHERE nid = :nid', array(':nid' => $nid))->fetchField();
    if (!$exists) {
      form_set_error('node', 'Invalid node selected.');
    }
  }
  else {
    form_set_error('node', 'No node selected.');
  }
  // Check node can be added.
  $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE ppt.related_nid = :nid AND pptt.tid = :tid AND ppt.' . db_escape_table($field) . ' = 1';
  $exists = db_query($sql, array(':nid' => $nid, ':tid' => $tid))->fetchField();
  if ($exists) {
    form_set_error('node', 'Node already added to promo block.');
  }
}

/**
 * Process add node form submission.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_add_node_to_block_submit($form, &$state) {
  // Get parameters.
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  preg_match('/\([0-9]+\)$/', $state['values']['node'], $match);
  $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
  // Check if the node has a teaser and get the latest one.
  $teaser_id = db_query('SELECT MAX(teaser_id) FROM {promo_picker_teasers} WHERE related_nid = :nid', array(':nid' => $nid))->fetchField();
  // Up date teaser record or create new one.
  if ($teaser_id) {
    $teaser = db_query('SELECT * FROM {promo_picker_teasers} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser_id))->fetchObject();
    $teaser->{$field} = 1;
    drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
    $assigned = db_query('SELECT COUNT(teaser_id) FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = :teaser AND tid = :tid', array(':teaser' => $teaser->teaser_id, ':tid' => $tid))->fetchField();
    if (!$assigned) {
      $tax = new stdClass();
      $tax->teaser_id = $teaser->teaser_id;
      $tax->tid = $tid;
      drupal_write_record('promo_picker_teasers_taxonomy', $tax);
    }
  }
  else {
    $node = db_query('SELECT * FROM {node} WHERE nid = :nid', array(':nid' => $nid))->fetchObject();
    $teaser = new stdClass();
    $teaser->title = $node->title;
    $teaser->body = '';
    $teaser->related_nid = $nid;
    $teaser->{$field} = 1;
    $teaser->uid = $GLOBALS['user']->uid;
    drupal_write_record('promo_picker_teasers', $teaser);
    $tax = new stdClass();
    $tax->teaser_id = $teaser->teaser_id;
    $tax->tid = $tid;
    drupal_write_record('promo_picker_teasers_taxonomy', $tax);
  }
  $query = (!empty($_GET['return_to'])) ? array('query' => array('return_to' => $_GET['return_to'])) : array();
  drupal_goto('admin/structure/promo_picker/blocks/edit/' . $tid . '/' . $block, $query);
}

/**
 * Validate promo block edit form.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_blocks_admin_edit_form_validate($form, &$state) {
  // Nothing to validate.
}

/**
 * Process promo block edit form submission.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_blocks_admin_edit_form_submit($form, &$state) {
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  foreach ($state['values']['teasers'] as $teaser_id => $data) {
    // Change teaser record for taxonomy.
    if ($teaser_id != $data['teaser_select']) {
      db_query('DELETE FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = :teaser_id AND tid = :tid', array(':teaser_id' => $teaser_id, ':tid' => $tid));
      $teaser = db_query('SELECT * FROM {promo_picker_teasers} WHERE teaser_id = :teaser_id', array(':teaser_id' => $data['teaser_select']))->fetchObject();
      $teaser->{$field} = 1;
      drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
      $tax = new stdClass();
      $tax->teaser_id = $teaser->teaser_id;
      $tax->tid = $tid;
      drupal_write_record('promo_picker_teasers_taxonomy', $tax);
      $teaser_id = $teaser->teaser_id;
    }
    // Update teaser weight in promo block.
    $delta_field = ($block == 'primary') ? 'primary_delta' : 'secondary_delta';
    $taxonomy = db_query('SELECT * FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = :teaser_id AND tid = :tid', array(':teaser_id' => $teaser_id, ':tid' => $tid))->fetchObject();
    $taxonomy->{$delta_field} = $data['weight'];
    drupal_write_record('promo_picker_teasers_taxonomy', $taxonomy, array('teaser_id', 'tid'));
  }
  // Update block theme.
  if (!empty($state['values']['theme'])) {
    variable_set('promo_picker_theme_' . $tid . '_' . $block, $state['values']['theme']);
  }
  else {
    variable_del('promo_picker_theme_' . $tid . '_' . $block);
  }
  drupal_set_message(t('Promo block details saved.'));
  // Set page to return to.
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/structure/promo_picker/blocks';
  drupal_goto($goto);
}

/**
 * Remove teaser record from promo block.
 */
function node_promo_picker_block_remove_teaser() {
  $teaser = arg(5);
  $tid = arg(6);
  $block = arg(7);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.teaser_id = :teaser_id AND pptt.tid = :tid AND ppt.' . db_escape_table($field) . ' = 1';
  db_query($sql, array(':teaser_id' => $teaser, ':tid' => $tid));
  drupal_set_message(t('Teaser removed from block.'));
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/structure/promo_picker/blocks/edit/' . $tid . '/' . $block;
  $query = (!empty($_GET['referer'])) ? array('query' => array('return_to' => $_GET['referer'])) : array();
  drupal_goto($goto, $query);
}

/**
 * Theme teaser list into draggable table.
 */
function theme_promo_block_teaser_list($element) {
  $rows = array();
  foreach (element_children($element['element']) as $key) {
    if (isset($element['element'][$key]['node'])) {
      $teaser = &$element['element'][$key];
      $row = array();
      $row[] = drupal_render($teaser['node']);
      $row[] = drupal_render($teaser['teaser_select']);
      $teaser['weight']['#attributes']['class'] = array('block-teaser-weight');
      $row[] = drupal_render($teaser['weight']);
      $row[] = drupal_render($teaser['edit']);
      $row[] = drupal_render($teaser['new']);
      $row[] = drupal_render($teaser['remove']);
      $rows[] = array('data' => $row, 'class' => array('draggable'));
    }
  }
  if (empty($rows)) {
    $rows[] = array(array('data' => t('No teasers available.'), 'colspan' => '6'));
  }
  $header = array('Node', 'Teaser');
  $header[] = 'Weight';
  drupal_add_tabledrag('block-teaser-list', 'order', 'sibling', 'block-teaser-weight');
  $header[] = array('data' => 'Operations', 'colspan' => '3');
  return theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => array('id' => 'block-teaser-list')));
}
