<?php
/**
 * @file
 * Administration pages for managing teaser records.
 */

/**
 * Form definition for teaser list screen.
 *
 * @see node_promo_picker_teasers_admin_list_form_submit()
 */
function node_promo_picker_teasers_admin_list_form($state) {
  $form = array();
  $form['search_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Filter list',
    '#collapsible' => TRUE,
    '#collapsed' => !variable_get('promo_picker_list_filtered_' . $GLOBALS['user']->uid, FALSE),
  );
  $form['search_panel']['node'] = array(
    '#type' => 'textfield',
    '#title' => 'Node',
    '#default_value' => variable_get('promo_picker_node_filter_' . $GLOBALS['user']->uid, ''),
  );
  $form['search_panel']['block'] = array(
    '#type' => 'select',
    '#title' => 'Promo block',
    '#options' => array(
      '' => 'All',
      'primary' => 'Primary',
      'secondary' => 'Secondary',
    ),
    '#default_value' => variable_get('promo_picker_block_filter_' . $GLOBALS['user']->uid, ''),
  );
  $form['search_panel']['buttons']['refine'] = array(
    '#type' => 'submit',
    '#value' => 'Refine',
  );
  $form['search_panel']['buttons']['reset'] = array(
    '#type' => 'submit',
    '#value' => 'Reset',
  );
  $query = db_select('promo_picker_teasers', 't');
  $query = $query->extend('PagerDefault')->limit(50); // Tell Drupal to provide pagination information on the query results, 50 per page.
  $query->join('node', 'n', 't.related_nid = n.nid');
  $query->join('users', 'u', 't.uid = u.uid');
  $query->addField('n', 'title', 'node_title');
  $query
    ->fields('t', array('teaser_id', 'title', 'primary_flag', 'secondary_flag' ))
    ->fields('n', array('nid'))
    ->fields('u', array('name'))
    ->orderBy('t.teaser_id', 'DESC')
    ->addTag('node_access'); // Tell Drupal to add node access permission filters to this query.
  node_promo_picker_modify_teaser_filter_query($query);
  $result = $query->execute();
  $headers = array(
    'Teaser',
    'Node',
    'Primary',
    'Secondary',
    'User',
    array(
      'data' => 'Operations',
      'colspan' => 2,
    ),
  );
  $rows = array();
  foreach ($result as $row) {
    $rows[] = array(
      $row->title,
      $row->node_title . ' (NID: ' . $row->nid . ')',
      (($row->primary_flag) ? 'Yes' : 'No'),
      (($row->secondary_flag) ? 'Yes' : 'No'),
      $row->name,
      l(t('Edit'), 'admin/structure/promo_picker/edit-teaser/' . $row->teaser_id, array('query' => array('return_to' => 'admin/structure/promo_picker/teasers'))),
      l(t('Delete'), 'admin/structure/promo_picker/delete-teaser/' . $row->teaser_id, array('query' => array('return_to' => 'admin/structure/promo_picker/teasers'))),
    );
  }
  $form['table'] = array(
    '#element' => array(
      'rows' => array('#value' => $rows),
      'header' => array('#value' => $headers),
      'pager' => theme('pager'),
    ),
    '#theme' => 'teaser_list_table',
  );
  return $form;
}

/**
 * Process submission of search filter form
 *
 * @see node_promo_picker_teasers_admin_list_form()
 */
function node_promo_picker_teasers_admin_list_form_submit($form, &$state) {
  variable_del('promo_picker_list_filtered_' . $GLOBALS['user']->uid);
  if ($state['values']['op'] == 'Reset') {
    unset($state['values']['node']);
    unset($state['values']['taxonomy']);
    unset($state['values']['block']);
  }
  if (!empty($state['values']['node'])) {
    variable_set('promo_picker_node_filter_' . $GLOBALS['user']->uid, $state['values']['node']);
    variable_set('promo_picker_list_filtered_' . $GLOBALS['user']->uid, TRUE);
  }
  else {
    variable_del('promo_picker_node_filter_' . $GLOBALS['user']->uid);
  }
  if (!empty($state['values']['block'])) {
    variable_set('promo_picker_block_filter_' . $GLOBALS['user']->uid, $state['values']['block']);
    variable_set('promo_picker_list_filtered_' . $GLOBALS['user']->uid, TRUE);
  }
  else {
    variable_del('promo_picker_block_filter_' . $GLOBALS['user']->uid);
  }
}

/**
 * Delete teaser record.
 */
function node_promo_picker_teasers_admin_delete() {
  $teaser = arg(4);
  db_query('DELETE FROM {promo_picker_teasers} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser));
  db_query('DELETE FROM {promo_picker_teasers_images} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser));
  db_query('DELETE FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser));
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/structure/promo_picker/teasers';
  drupal_set_message(t('Teaser record deleted.'));
  drupal_goto($goto);
}

/**
 * Takes a reference to a Drupal 7 SelectQuery object and adds any relevant
 * filters to it.
 * 
 * @param object $query
 *   A Drupal 7 SelectQuery object, such as that created by db_select(). 
 */
function node_promo_picker_modify_teaser_filter_query(&$query) {
  // Check the node filter.
  $node_filter = variable_get('promo_picker_node_filter_' . $GLOBALS['user']->uid, '');
  if (!empty($node_filter)) {
    $query->condition(
      db_or()
        ->condition('n.title', '%' . $node_filter . '%', 'LIKE')
        ->condition('n.nid', $node_filter));
  }
  // Check the block filter.
  $block_filter = variable_get('promo_picker_block_filter_' . $GLOBALS['user']->uid, '');
  if (!empty($block_filter)) {
    if ($block_filter == 'primary') {
      $query->condition('t.primary_flag', 1);
    }
    elseif ($block_filter == 'secondary') {
      $query->condition('t.secondary_flag', 1);
    }
  }
}


/**
 * Form definition for add new node teaser.
 *
 * @see node_promo_picker_teasers_admin_add_form_validate()
 * @see node_promo_picker_teasers_admin_add_form_submit()
 */
function node_promo_picker_teasers_admin_add_form($state) {
  $form = array();
  // Node autocomplete search field.
  $form['node'] = array(
    '#type' => 'textfield',
    '#title' => 'Node',
    '#description' => 'Search for node to create teaser for.',
    '#autocomplete_path' => 'admin/structure/promo_picker/node/autocomplete',
  );
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Create teaser'),
  );
  return $form;
}

/**
 * Validate add teaser form submission.
 *
 * @see node_promo_picker_teasers_admin_add_form()
 */
function node_promo_picker_teasers_admin_add_form_validate($form, &$state) {
  if (preg_match('/\([0-9]+\)$/', $state['values']['node'], $match)) {
    $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
    $exists = db_query('SELECT nid FROM {node} WHERE nid = :nid', array(':nid' => $nid))->fetchField();
    if (!$exists) {
      form_set_error('node', 'Invalid node selected.');
    }
  }
  else {
    form_set_error('node', 'No node selected to create teaser for.');
  }
}

/**
 * Process add teaser form submission.
 *
 * @see node_promo_picker_teasers_admin_add_form()
 */
function node_promo_picker_teasers_admin_add_form_submit($form, &$state) {
  if (preg_match('/\([0-9]+\)$/', $state['values']['node'], $match)) {
    $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
    $query = array('query' => array('return_to' => 'admin/structure/promo_picker/add-teaser'));
    drupal_goto('admin/structure/promo_picker/create-teaser/' . $nid, $query);
  }
}

/**
 * Form definition for create a new teaser record.
 *
 * @see node_promo_picker_teasers_admin_create_form_validate()
 * @see node_promo_picker_teasers_admin_create_form_submit()
 */
function node_promo_picker_teasers_admin_create_form($state) {
  $form = array();
  // Get arguments from url.
  $nid = arg(4);
  $arg_tid = arg(5);
  $arg_block = arg(6);
  // Get node title.
  $title = db_query('SELECT title FROM {node} WHERE nid = :nid', array(':nid' => $nid))->fetchField();
  // Add teaser fields.
  $form['teaser_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Teaser',
  );
  $form['teaser_panel']['title'] = array(
    '#type' => 'textfield',
    '#title' => 'Title',
    '#default_value' => $title,
  );
  $form['teaser_panel']['teaser'] = array(
    '#type' => 'textarea',
    '#title' => 'Teaser',
  );
  $form['teaser_panel']['block'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Block',
    '#description' => 'Specify promo block to display teaser in.',
    '#options' => array('primary' => 'Primary', 'secondary' => 'Secondary'),
    '#default_value' => ((!empty($arg_block)) ? array($arg_block) : array()),
  );
  $image_node = variable_get('promo_picker_image_node', NULL);
  if (!empty($image_node)) {
    // Add image fields.
    $form['image_panel'] = array(
      '#type' => 'fieldset',
      '#title' => 'Images',
    );
    $form['image_panel']['images'] = array(
      '#tree' => TRUE,
      '#prefix' => '<div id="teaser-images">',
      '#suffix' => '</div>',
      '#theme' => 'form_teaser_images',
    );
    $num_images = variable_get('promo_picker_max_images', 3);
    if ($num_images == 0) {
      $form['image_panel']['images'][] = array(
        'image' => array(
          '#type' => 'textfield',
          '#title' => '',
          '#autocomplete_path' => 'admin/structure/promo_picker/image/autocomplete',
          '#default_value' => '',
        ),
        'weight' => array(
          '#type' => 'weight',
          '#default_value' => 0,
          '#delta' => 10,
        ),
      );
      $form['image_panel']['more'] = array(
        '#type' => 'submit',
        '#value' => 'Add another image',
        '#ajax' => array(
          'path' => 'admin/structure/promo_picker/images/ajax',
          'wrapper' => 'teaser-images',
          'method' => 'replace',
          'effect' => 'fade',
        ),
      );
    }
    else {
      for ($i = 0; $i < $num_images; $i++) {
        $form['image_panel']['images'][] = array(
          'image' => array(
            '#type' => 'textfield',
            '#title' => 'Image',
            '#autocomplete_path' => 'admin/structure/promo_picker/image/autocomplete',
          ),
          'weight' => array(
            '#type' => 'weight',
            '#default_value' => 0,
            '#delta' => 10,
          ),
        );
      }
    }
  }
  // Add taxonomy fields.
  $form['taxonomy_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Taxonomy',
  );
  $vids = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($vids as $vid) {
    $vname = db_query('SELECT name FROM {taxonomy_vocabulary} WHERE vid = :vid', array(':vid' => $vid))->fetchField();
    $form['taxonomy_panel']['vocabulary_' . $vid] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($vname),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $options = array();
    $terms = taxonomy_get_tree($vid);
    foreach ($terms as $t) {
      $options[$t->tid] = $t->name;
    }
    $form['taxonomy_panel']['vocabulary_' . $vid]['taxonomy_' . $vid] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => ((!empty($arg_tid) && in_array($arg_tid, array_keys($options))) ? array($arg_tid) : array()),
    );
  }
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save teaser'),
  );
  return $form;
}

/**
 * Validate teaser creation form submission.
 *
 * @see node_promo_picker_teasers_admin_create_form()
 */
function node_promo_picker_teasers_admin_create_form_validate($form, &$state) {
  if (!empty($state['ajax_submission'])) {
    return;
  }
  $post = $state['values'];
  // Validate teaser title.
  if (empty($post['title'])) {
    form_set_error('title', 'No teaser title entered.');
  }
  // Validate teaser text.
  if (empty($post['teaser'])) {
    form_set_error('teaser', 'No teaser text entered.');
  }
  // Validate image nodes if selected.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $i) {
      if (!empty($i['image'])) {
        if (preg_match('/\([0-9]+\)$/', $i['image'], $match)) {
          $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          $exists = db_query('SELECT nid FROM {node} WHERE nid = :nid', array(':nid' => $nid))->fetchField();
          if (!$exists) {
            form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
          }
        }
        else {
          form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
        }
      }
    }
  }
}

/**
 * Process teaser creation form submission.
 *
 * @see node_promo_picker_teasers_admin_create_form()
 */
function node_promo_picker_teasers_admin_create_form_submit($form, &$state) {
  if (!empty($state['ajax_submission'])) {
    return;
  }
  $nid = arg(4);
  $post = $state['values'];
  // Create teaser record.
  $teaser = new stdClass();
  $teaser->title = $post['title'];
  $teaser->body = $post['teaser'];
  $teaser->related_nid = $nid;
  $teaser->primary_flag = (!empty($post['block']['primary'])) ? 1 : 0;
  $teaser->secondary_flag = (!empty($post['block']['secondary'])) ? 1 : 0;
  $teaser->uid = $GLOBALS['user']->uid;
  drupal_write_record('promo_picker_teasers', $teaser);
  // Create taxonomy records.
  $taxonomies = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($taxonomies as $vid) {
    if (!empty($post['taxonomy_' . $vid])) {
      foreach ($post['taxonomy_' . $vid] as $tid) {
        if ($tid) {
          // Get taxonomy name.
          $tname = db_query('SELECT name FROM {taxonomy_term_data} WHERE tid = :tid', array(':tid' => $tid))->fetchField();
          // Check if max teasers assigned to taxonomy.
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = :tid AND ppt.primary_flag = 1';
          $count = db_query($sql, array(':tid' => $tid))->fetchField();
          if ($count >= variable_get('promo_picker_max_primary_teasers', 3) && $teaser->primary_flag) {
            $teaser->primary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname primary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = :tid AND ppt.secondary_flag = 1';
          $count = db_query($sql, array(':tid' => $tid))->fetchField();
          if ($count >= variable_get('promo_picker_max_secondary_teasers', 3) && $teaser->secondary_flag) {
            $teaser->secondary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname secondary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          // Ensure node only has 1 teaser per taxonomy, per block.
          if ($teaser->primary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = :related_nid AND pptt.tid = :tid AND ppt.primary_flag = 1';
            db_query($sql, array(':related_nid' => $teaser->related_nid, ':tid' => $tid));
          }
          if ($teaser->secondary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = :related_nid AND pptt.tid = :tid AND ppt.secondary_flag = 1';
            db_query($sql, array(':related_nid' => $teaser->related_nid, ':tid' => $tid));
          }
          // Create taxonomy record.
          $tax = new stdClass();
          $tax->teaser_id = $teaser->teaser_id;
          $tax->tid = $tid;
          $tax->primary_delta = 0;
          $tax->secondary_delta = 0;
          drupal_write_record('promo_picker_teasers_taxonomy', $tax);
        }
      }
    }
  }
  // Remove currently assign images.
  db_query('DELETE FROM {promo_picker_teasers_images} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser->teaser_id));
  // Create image records.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $delta => $img) {
      if (!empty($img['image'])) {
        if (preg_match('/\([0-9]+\)$/', $img['image'], $match)) {
          $inid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          // Insert image record.
          $image = new stdClass();
          $image->teaser_id = $teaser->teaser_id;
          $image->image_nid = $inid;
          $image->delta = $img['weight'];
          drupal_write_record('promo_picker_teasers_images', $image);
        }
      }
    }
  }
  // Clear form cache.
  cache_clear_all('form_' . $state['values']['form_build_id'], 'cache_form');
  cache_clear_all('storage_' . $state['values']['form_build_id'], 'cache_form');
  // Set page to return to.
  drupal_set_message(t('Teaser record created.'));
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/structure/promo_picker/add-teaser';
  $query = (!empty($_GET['referer'])) ? array('query' => array('return_to' => $_GET['referer'])) : array();
  drupal_goto($goto, $query);
}

/**
 * Form definition for edit teaser record.
 *
 * @see node_promo_picker_teasers_admin_edit_form_validate()
 * @see node_promo_picker_teasers_admin_edit_form_submit()
 */
function node_promo_picker_teasers_admin_edit_form($form, &$state) {
  $form = array();
  // Get teaser record.
  $teaser_id = (!empty($state['values']['teaser_id']) ? $state['values']['teaser_id'] : arg(4));
  $record = db_query('SELECT * FROM {promo_picker_teasers} WHERE teaser_id = :teaser_id', array('teaser_id' => $teaser_id))->fetchObject();
  if (empty($record)) {
    drupal_set_message(t('Invalid teaser record - @teaserid', array('@teaserid' => $teaser_id)), 'error');
    return $form;
  }
  // Get node.
  $node = db_query('SELECT n.*, t.name FROM {node} AS n JOIN {node_type} AS t ON n.type = t.type WHERE n.nid = :nid', array(':nid' => $record->related_nid))->fetchObject();
  // Add node details.
  $form['teaser_id'] = array(
    '#type' => 'hidden',
    '#value' => $teaser_id,
  );
  $form['node_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Node',
  );
  $form['node_panel']['node_id'] = array(
    '#type' => 'item',
    '#title' => 'NID',
    '#markup' => $node->nid,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['node_panel']['node_type'] = array(
    '#type' => 'item',
    '#title' => 'Type',
    '#markup' => $node->name,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['node_panel']['node_title'] = array(
    '#type' => 'item',
    '#title' => 'Title',
    '#markup' => $node->title,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  // Add teaser fields.
  $form['teaser_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Teaser',
  );
  $form['teaser_panel']['title'] = array(
    '#type' => 'textfield',
    '#title' => 'Title',
    '#default_value' => $record->title,
  );
  $form['teaser_panel']['teaser'] = array(
    '#type' => 'textarea',
    '#title' => 'Teaser',
    '#default_value' => $record->body,
  );
  $block = array();
  if ($record->primary_flag) {
    $block[] = 'primary';
  }
  if ($record->secondary_flag) {
    $block[] = 'secondary';
  }
  $form['teaser_panel']['block'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Block',
    '#description' => 'Specify promo block to display teaser in.',
    '#options' => array('primary' => 'Primary', 'secondary' => 'Secondary'),
    '#default_value' => $block,
  );
  $image_node = variable_get('promo_picker_image_node', NULL);
  if (!empty($image_node)) {
    // Get image nodes for teaser.
    $images = array();
    $result = db_query('SELECT n.title, ti.image_nid, ti.delta FROM {node} AS n JOIN {promo_picker_teasers_images} AS ti ON n.nid = ti.image_nid WHERE ti.teaser_id = :teaser_id ORDER BY ti.delta ASC', array(':teaser_id' => $teaser_id));
    foreach ($result as $img) {
      $images[] = $img;
    }
    // Add image fields.
    $form['image_panel'] = array(
      '#type' => 'fieldset',
      '#title' => 'Images',
    );
    $form['image_panel']['images'] = array(
      '#tree' => TRUE,
      '#prefix' => '<div id="teaser-images">',
      '#suffix' => '</div>',
      '#theme' => 'form_teaser_images',
    );
    $num_images = variable_get('promo_picker_max_images', 3);
    foreach ($images as $i) {
      $form['image_panel']['images'][] = array(
        'image' => array(
          '#type' => 'textfield',
          '#title' => '',
          '#autocomplete_path' => 'admin/structure/promo_picker/image/autocomplete',
          '#default_value' => $i->title . ' (' . $i->image_nid . ')',
        ),
        'weight' => array(
          '#type' => 'weight',
          '#default_value' => $i->delta,
          '#delta' => 10,
        ),
      );
    }
    for ($i = count($images); $i < $num_images; $i++) {
      $form['image_panel']['images'][] = array(
        'image' => array(
          '#type' => 'textfield',
          '#title' => '',
          '#autocomplete_path' => 'admin/structure/promo_picker/image/autocomplete',
        ),
        'weight' => array(
          '#type' => 'weight',
          '#default_value' => $i,
          '#delta' => 10,
        ),
      );
    }
    if ($num_images == 0) {
      $num_images_ajax = (!empty($state['values']['images'])) ? (count($state['values']['images']) + 1) - count($images) : 1;
      for ($i = 0; $i < $num_images_ajax; $i++) {
        $form['image_panel']['images'][] = array(
          'image' => array(
            '#type' => 'textfield',
            '#title' => '',
            '#autocomplete_path' => 'admin/structure/promo_picker/image/autocomplete',
          ),
          'weight' => array(
            '#type' => 'weight',
            '#default_value' => 0,
            '#delta' => 10,
          ),
        );
      }
      $form['image_panel']['more'] = array(
        '#type' => 'submit',
        '#value' => 'Add another image',
        '#submit' => array('node_promo_picker_teasers_admin_edit_form_add_another_image_add_one'),
        '#ajax' => array(
          'callback' => 'node_promo_picker_teasers_admin_edit_form_add_another_image_callback',
          'wrapper' => 'teaser-images',
          'method' => 'replace',
          'effect' => 'fade',
        ),
      );
    }
  }
  // Get teaser taxonomy association.
  $teaser_tids = array();
  $result = db_query('SELECT tid FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser_id));
  while ($t = $result->fetchField()) {
    $teaser_tids[] = $t;
  }
  // Add taxonomy fields.
  $form['taxonomy_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Taxonomy',
  );
  $taxonomies = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($taxonomies as $vid) {
    $vname = db_query('SELECT name FROM {taxonomy_vocabulary} WHERE vid = :vid', array(':vid' => $vid))->fetchField();
    $form['taxonomy_panel']['vocabulary_' . $vid] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($vname),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $options = array();
    $terms = taxonomy_get_tree($vid);
    foreach ($terms as $t) {
      $options[$t->tid] = $t->name;
    }
    $form['taxonomy_panel']['vocabulary_' . $vid]['taxonomy_' . $vid] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => $teaser_tids,
    );
  }
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save teaser'),
  );
  return $form;
}

/**
 * AJAX callback function for adding another row of image fields to the form.
 */
function node_promo_picker_teasers_admin_edit_form_add_another_image_callback($form, &$form_state) {
  return $form['image_panel']['images'];
}

/**
 * Forces the form to rebuild itself, adding one more row of image fields.
 */
function node_promo_picker_teasers_admin_edit_form_add_another_image_add_one($form, &$form_state) {
  $form_state['rebuild'] = TRUE;
}

/**
 * Validate teaser edit form submission.
 *
 * @see node_promo_picker_teasers_admin_edit_form()
 */
function node_promo_picker_teasers_admin_edit_form_validate($form, &$state) {
  if (!empty($state['triggering_element']['#ajax'])) {
    return;
  }
  $post = $state['values'];
  // Validate teaser title.
  if (empty($post['title'])) {
    form_set_error('title', 'No teaser title entered.');
  }
  // Validate teaser text.
  if (empty($post['teaser'])) {
    form_set_error('teaser', 'No teaser text entered.');
  }
  // Validate image nodes if selected.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $i) {
      if (!empty($i['image'])) {
        if (preg_match('/\([0-9]+\)$/', $i['image'], $match)) {
          $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          $exists = db_query('SELECT nid FROM {node} WHERE nid = :nid', array(':nid' => $nid))->fetchField();
          if (!$exists) {
            form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
          }
        }
        else {
          form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
        }
      }
    }
  }
}

/**
 * Process teaser creation form submission.
 *
 * @see node_promo_picker_teasers_admin_edit_form()
 */
function node_promo_picker_teasers_admin_edit_form_submit($form, &$state) {
  if (!empty($state['triggering_element']['#ajax'])) {
    return;
  }
  $teaser_id = (!empty($state['values']['teaser_id']) ? $state['values']['teaser_id'] : arg(4));
  $post = $state['values'];
  // Create teaser record.
  $teaser = db_query('SELECT * FROM {promo_picker_teasers} WHERE teaser_id = :teaser_id', array('teaser_id' => $teaser_id))->fetchObject();
  $teaser->teaser_id = $teaser_id;
  $teaser->title = $post['title'];
  $teaser->body = $post['teaser'];
  $teaser->primary_flag = (!empty($post['block']['primary'])) ? 1 : 0;
  $teaser->secondary_flag = (!empty($post['block']['secondary'])) ? 1 : 0;
  drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
  // Get teaser taxonomy records, to check if any need deleting.
  $result = db_query('SELECT tid FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser->teaser_id));
  $teaser_tids = array();
  while ($row = $result->fetchField()) {
    $teaser_tids[$row] = $row;
  }
  // Create taxonomy records.
  $taxonomies = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($taxonomies as $vid) {
    if (!empty($post['taxonomy_' . $vid])) {
      foreach ($post['taxonomy_' . $vid] as $tid) {
        if ($tid) {
          // Remove tid from array so it is not deleted.
          if (!empty($teaser_tids[$tid])) {
            unset($teaser_tids[$tid]);
          }
          // Check if taxonomy record already exists, no need to update it.
          if (db_query('SELECT teaser_id FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = :teaser_id AND tid = :tid', array(':teaser_id' => $teaser->teaser_id, ':tid' => $tid))->fetchField()) {
            continue;
          }
          // Get taxonomy name.
          $tname = db_query('SELECT name FROM {taxonomy_term_data} WHERE tid = :tid', array(':tid' => $tid))->fetchField();
          // Check if max teasers assigned to taxonomy.
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = :tid AND ppt.primary_flag = 1';
          $count = db_query($sql, array(':tid' => $tid))->fetchField();
          if ($count >= variable_get('promo_picker_max_primary_teasers', 3) && $teaser->primary_flag) {
            $teaser->primary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname primary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = :tid AND ppt.secondary_flag = 1';
          $count = db_query($sql, array(':tid' => $tid))->fetchField();
          if ($count >= variable_get('promo_picker_max_secondary_teasers', 3) && $teaser->secondary_flag) {
            $teaser->secondary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname secondary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          // Ensure node only has 1 teaser per taxonomy, per block.
          if ($teaser->primary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = :related_nid AND pptt.tid = :tid AND ppt.primary_flag = 1';
            db_query($sql, array(':related_nid' => $teaser->related_nid, ':tid' => $tid));
          }
          if ($teaser->secondary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = :related_nid AND pptt.tid = :tid AND ppt.secondary_flag = 1';
            db_query($sql, array(':related_nid' => $teaser->related_nid, ':tid' => $tid));
          }
          // Create taxonomy record.
          $tax = new stdClass();
          $tax->teaser_id = $teaser->teaser_id;
          $tax->tid = $tid;
          $tax->primary_delta = 0;
          $tax->secondary_delta = 0;
          drupal_write_record('promo_picker_teasers_taxonomy', $tax);
        }
      }
    }
  }
  // Delete any removed taxonomies.
  if (!empty($teaser_tids)) {
    foreach ($teaser_tids as $rtid) {
      db_query('DELETE FROM {promo_picker_teasers_taxonomy} WHERE tid = :tid AND teaser_id = :teaser_id', array(':tid' => $rtid, ':teaser_id' => $teaser->teaser_id));
    }
  }
  // Remove currently assign images.
  db_query('DELETE FROM {promo_picker_teasers_images} WHERE teaser_id = :teaser_id', array(':teaser_id' => $teaser->teaser_id));
  // Create image records.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $delta => $img) {
      if (!empty($img['image'])) {
        if (preg_match('/\([0-9]+\)$/', $img['image'], $match)) {
          $inid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          // Insert image record.
          $image = new stdClass();
          $image->teaser_id = $teaser->teaser_id;
          $image->image_nid = $inid;
          $image->delta = $img['weight'];
          drupal_write_record('promo_picker_teasers_images', $image);
        }
      }
    }
  }
  // Clear form cache.
  cache_clear_all('form_' . $state['values']['form_build_id'], 'cache_form');
  cache_clear_all('storage_' . $state['values']['form_build_id'], 'cache_form');
  // Set page to return to.
  drupal_set_message(t('Teaser record updated.'));
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/structure/promo_picker/teasers';
  $query = (!empty($_GET['referer'])) ? array('query' => array('return_to' => $_GET['referer'])) : array();
  drupal_goto($goto, $query);
}

/**
 * Theme the images field in teaser creation/edit forms.
 */
function theme_form_teaser_images($element) {
  $element = $element['element'];
  foreach (element_children($element) as $key) {
    if (isset($element[$key]['image'])) {
      $image = &$element[$key];
      $row = array();
      $row[] = drupal_render($image['image']);
      $image['weight']['#attributes']['class'] = array('image-weight');
      $row[] = drupal_render($image['weight']);
      $rows[] = array('data' => $row, 'class' => array('draggable'));
    }
  }
  $header = array('Image', 'Weight');
  drupal_add_tabledrag('image-list', 'order', 'sibling', 'image-weight');
  return theme('table', array(
    'header' => $header,
    'rows' => (isset($rows) ? $rows : array()),
    'attributes' => array('id' => 'image-list')));
}

/**
 * Theme the teaser list table.
 */
function theme_teaser_list_table($element) {
  if (!empty($element['element']['rows']['#value'])) {
    $rows = $element['element']['rows']['#value'];
  }
  else {
    $rows[] = array(
      array(
        'data' => t('No posts available.'),
        'colspan' => '7',
      ),
    );
  }
  $output = theme(
    'table',
    array(
      'header' => $element['element']['header']['#value'],
      'rows' => $rows,
    )
  );
  
  if ($element['element']['pager']) {
    $output .= $element['element']['pager'];
  }
  
  return $output;
}
